@@ -4,84 +4,85 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include "uqwk.h"
+#ifdef	GNKSA
+	#include <gnksa.h>
+#endif
+
 /*
  *  Process a reply packet
  */
 
-DoReply ()
+
+int DoReply ()
 {
-	int n, rep_cnt;
+	int n, rep_cnt, ret;
 	char bbs[PATH_LEN];
 
 	/* Check for ZipNews reply */
 	if (zip_mode)
-	{
-		DoZipReplies();
-		return (0);
-	}
+		return DoZipReplies();
 
 	rep_cnt = 0;
 
 	/* Open the packet */
-	if (NULL == (rep_fd = fopen (rep_file, "r")))
-	{
+	if (NULL == (rep_fd = fopen (rep_file, "r"))) {
 		fprintf (stderr, "%s: can't open %s\n", progname, rep_file);
-		return (0);
+		return(-1);
 	}
 
 	/* Handle SLNP reply */
-	if (slnp_mode)
-	{
-		SLNPReply ();
+	if (slnp_mode) {
+		if((ret=SLNPReply()))
+			return ret;
+
 		fclose (rep_fd);
-		return (0);
+		return 0;
 	}
 
 	/* Get the first block, the BBS ID */
-	if (1 != fread (buf, 128, 1, rep_fd))
-	{
+	if (1 != fread (buf, 128, 1, rep_fd)) {
 		fprintf (stderr, "%s: reply packet read error\n", progname);
 		fclose (rep_fd);
-		return (0);
+		return -1;
 	}
 
 	/* Extract bbs id and check */
 	sscanf (bbs_id, "%*d,%s", bbs);
 	n = strlen (bbs);
 	buf[n] = 0;
-	if (strcmp (bbs, buf))
-	{
+	if (strcmp (bbs, buf)) {
 		fprintf (stderr, "%s: reply BBS ID mismatch: %s != %s\n",
 			progname, buf, bbs);
 		fclose (rep_fd);
-		return (0);
+		return 0;
 	}
 
 	/* Read the .newsrc file; we will need the list of conferences */
-	ReadNewsrc();
+	if((ret=ReadNewsrc()<=0))
+		return(ret);
 
 	/* Read the next message header and process it */
-	while (1 == fread (&rep_hdr, 128, 1, rep_fd))
-	{
-		SendReply ();
+	while (1 == fread (&rep_hdr, 128, 1, rep_fd)) {
+		if((ret=SendReply()) <= 0) return(ret);
 		rep_cnt++;
 	}
 
 	fclose (rep_fd);
 	printf ("%s: sent %d replies\n", progname, rep_cnt);
 
-	return (1);
+	return 1;
 }
 
-SendReply ()
 /*
  *  Pipe a reply to the mailer or inews
  */
+int SendReply ()
 {
-	FILE *pfd;
-	unsigned char c, to[PATH_LEN], subject[PATH_LEN], group[PATH_LEN];
-	int i, n, blocks, bytes, conf;
-	struct nrc_ent *np;
+	FILE    *pfd;
+	unsigned char    c;
+        char    to[PATH_LEN], subject[PATH_LEN], group[PATH_LEN];
+	int     i, n, blocks, bytes, conf;
+	struct  nrc_ent *np;
 
 	/* Extract recipient */
 	strncpy (buf, rep_hdr.to, 25);
@@ -104,20 +105,16 @@ SendReply ()
 
 	/* Find newsgroup with this conference number */
 	np = nrc_list;
-	while (np != NULL)
-	{
+	while (np != NULL) {
 		if (np->conf == conf) break;
 		np = np->next;
 	}
 
 	/* Get newsgroup name */
-	if (np == NULL)
-	{
+	if (np == NULL) {
 		/* Bet this generates lots of email for "ALL" */
 		rep_hdr.status = QWK_PRIVATE;
-	}
-	else
-	{
+	} else {
 		strcpy (group, np->name);
 	}
 
@@ -130,27 +127,23 @@ SendReply ()
 
 	/* Check for off-line command message */
 	if ( (!strcmp (to, "uqwk")) || (!strcmp (to, "UQWK")) )
-	{
-		QWKOffLine (bytes, rep_fd);
-		return (0);
-	}
+		return(QWKOffLine (bytes, rep_fd));
 
 	/* Check for a configuration message intended for some
 	   other QWK "door" */
-	if ( (!strcmp (to, "MARKMAIL")) || (!strcmp (to, "QMAIL"))   ||
-	     (!strcmp (to, "markmail")) || (!strcmp (to, "qmail"))   ||
-	     (!strcmp (to, "ROSEMAIL")) || (!strcmp (to, "KMAIL"))   ||
-	     (!strcmp (to, "rosemail")) || (!strcmp (to, "kmail"))   ||
-	     (!strcmp (to, "MAINMAIL")) || (!strcmp (to, "CMPMAIL")) ||
-	     (!strcmp (to, "mainmail")) || (!strcmp (to, "cmpmail")) ||
-	     (!strcmp (to, "ULTRABBS")) || (!strcmp (to, "BGQWK"))   ||
-	     (!strcmp (to, "ultrabbs")) || (!strcmp (to, "bgqwk"))   ||
-	     (!strcmp (to, "CAM-MAIL")) || (!strcmp (to, "TRIMAIL")) ||
-	     (!strcmp (to, "cam-mail")) || (!strcmp (to, "trimail")) ||
-	     (!strcmp (to, "QSO")) || (!strcmp (to, "qso")) )
-	{
+	if (	(!strcasecmp (to, "MARKMAIL"))	||
+		(!strcasecmp (to, "QMAIL"))	||
+		(!strcasecmp (to, "ROSEMAIL"))	||
+		(!strcasecmp (to, "KMAIL"))	||
+		(!strcasecmp (to, "MAINMAIL"))	||
+		(!strcasecmp (to, "CMPMAIL"))	||
+		(!strcasecmp (to, "ULTRABBS"))	||
+		(!strcasecmp (to, "BGQWK"))	||
+		(!strcasecmp (to, "CAM-MAIL"))	||
+		(!strcasecmp (to, "TRIMAIL"))	||
+		(!strcasecmp (to, "QSO"))	) {
 		/* Send warning to user */
-		SendWarning (to);
+		if((n=SendWarning(to)) <= 0) return(n);
 
 		/* Skip the rest of the message */
 		while (bytes--) fread (&c, 1, 1, rep_fd);
@@ -164,72 +157,63 @@ SendReply ()
 	/* Open pipe to proper program */
 	pfd = NULL;
 
+/*
+ *	QWK is beyond hope -- don't even bother basic GNKSA-checking :-(
+ */
+
 	if ( (rep_hdr.status == QWK_PUBLIC) ||
-	     (rep_hdr.status == QWK_PUBLIC2) )
-	{
+	     (rep_hdr.status == QWK_PUBLIC2) ) {
 		/* Public message, open pipe to inews */
-		if (xprt_mode)
-		{
-			sprintf (buf, "%s", XPRT_INEWS);
-		}
-		else
-#ifdef SERVER
-		{
-			sprintf (buf, "%s", INEWS_PATH);
-		}
+		if (xprt_mode) {
+			sprintf (buf, "%s", posting_agent);
+		} else {
+#ifdef NNTP
+			sprintf (buf, "%s", inews_path);
 #else
-		{
 			sprintf (buf, "%s -t '%s' -n '%s'",
-				INEWS_PATH, subject, group);
-		}
+				inews_path, subject, group);
 #endif
-		printf ("%s: Posting to %s\n", progname, group);
-		if (NULL == (pfd = popen (buf, "w")))
-		{
-			fprintf (stderr, "%s: can't popen() inews\n",
-					progname);
 		}
-#ifdef SERVER
-		if (!xprt_mode && (pfd != NULL) )
-		{
+		printf ("%s: Posting to %s\n", progname, group);
+		if (NULL == (pfd = popen (buf, "w"))) {
+			fprintf (stderr, "%s: can't popen() %s\n",
+					progname, inews_path);
+			return(-1);
+		}
+#ifdef NNTP
+		if (!xprt_mode && (pfd != NULL) ) {
 			fprintf (pfd, "Newsgroups: %s\nSubject: %s\n\n",
 				group, subject);
 		}
 #endif
-	}
-	else if ( (rep_hdr.status == QWK_PRIVATE) ||
-	          (rep_hdr.status == QWK_PRIVATE2) )
-	{
+	} else if ( (rep_hdr.status == QWK_PRIVATE) ||
+	          (rep_hdr.status == QWK_PRIVATE2) ) {
 		/* Open pipe to mail */
-		if (xprt_mode)
-		{
+		if (xprt_mode) {
 			sprintf (buf, "%s", XPRT_MAILER);
-		}
-		else
-		{
-			sprintf (buf, "%s -s '%s' '%s'",
-				MAILER_PATH, subject, to);
+		} else {
+			sprintf (buf, "%s '%s'",
+				SENDMAIL, to);
 		}
 		printf ("%s: Mailing to %s\n", progname, to);
-		if (NULL == (pfd = popen (buf, "w")))
-		{
-			fprintf (stderr, "%s: can't popen() mail\n", progname);
+		if (NULL == (pfd = popen (buf, "w"))) {
+			fprintf (stderr, "%s: can't popen() %s\n", progname, buf);
+			return(-1);
 		}
+		fprintf(pfd,"Subject: %s\n", subject);
 	}
 
 	/* Read and send all bytes of message */
-	for (i=0; i<bytes; i++)
-	{
+	for (i=0; i<bytes; i++) {
 		fread (&c, 1, 1, rep_fd);
 		if (c == QWK_EOL) c = 012;
-		if (pfd != NULL) fwrite (&c, 1, 1, pfd);
+		if( (pfd != NULL) && (fwrite(&c, 1, 1, pfd) != 1)) return(-1);
 	}
 
-	if (pfd != NULL) pclose (pfd);
-	return (1);
+	return(pclose (pfd));
 }
 
-SendWarning (to)
+int SendWarning (to)
 char *to;
 /*
  *  Mail a warning to the user if the reply packet
@@ -240,36 +224,35 @@ char *to;
 	FILE *pfd;
 
 	/* Open pipe to mailer */
+/*
 	sprintf (buf, "%s -s 'UQWK Error Message' %s",
 			MAILER_PATH, user_name);
-	if (NULL == (pfd = popen (buf, "w")))
-	{
+*/
+	sprintf (buf, "%s %s",
+			SENDMAIL, user_name);
+	if (NULL == (pfd = popen (buf, "w"))) {
 		fprintf (stderr, "%s: can't popen() mail\n", progname);
-		return (0);
+		return(-1);
 	}
 
 	/* Send the message */
 
 	fprintf (pfd,
-"Hello. You sent a message to the username %s, presumably to\n", to);
-	fprintf (pfd,
-"perform some sort of offline configuration. This QWK processor,\n");
-	fprintf (pfd,
-"called UQWK, cannot process this message. To perform offline\n");
-	fprintf (pfd,
-"configuration using UQWK, you must send a message to the username\n");
-	fprintf (pfd,
-"UQWK. Commands are to be included in the body of the message.\n");
-	fprintf (pfd,
-"For a list of commands, send a message to UQWK with the word\n");
-	fprintf (pfd,
-"HELP in the body of the message (not the subject). Thanks!\n");
+		"Subject: UQWK Error Message\n\n"
 
-	pclose (pfd);
-	return (1);
+		"Hello. You sent a message to the username %s, presumably to\n"
+		"perform some sort of offline configuration. This QWK processor,\n"
+		"called UQWK, cannot process this message. To perform offline\n"
+		"configuration using UQWK, you must send a message to the username\n"
+		"UQWK. Commands are to be included in the body of the message.\n"
+		"For a list of commands, send a message to UQWK with the word\n"
+		"HELP in the body of the message (not the subject). Thanks!\n",
+		to);
+
+	return(pclose (pfd));
 }
 
-CheckTo (to, bytes)
+int CheckTo (to, bytes)
 char *to;
 int bytes;
 /*
@@ -292,8 +275,7 @@ int bytes;
 	/* Check first four bytes */
 	fread (buf, 4, 1, rep_fd);
 	bytes -= 4;
-	if (strncmp (buf, "To: ", 4))
-	{
+	if (strncmp (buf, "To: ", 4)) {
 		/* Doesn't match */
 		fseek (rep_fd, offset, 0);
 		return (0);
@@ -306,8 +288,7 @@ int bytes;
 	bytes--;
 
 	while ( (bytes >= 0) && (c != QWK_EOL) &&
-	        (c != 9) && (c != ' ') )
-	{
+	        (c != 9) && (c != ' ') ) {
 		to[i++] = c;
 
 		fread (&c, 1, 1, rep_fd);
@@ -320,26 +301,24 @@ int bytes;
 	return (1);
 }
 
-SLNPReply ()
 /*
  *  Process an SLNP reply packet
  */
+int SLNPReply ()
 {
 	char fname[PATH_LEN], kind[PATH_LEN], type[PATH_LEN];
+	int rval=0;
 
 	/* Look through lines in REPLIES file */
-	while (Fgets (buf, BUF_LEN, rep_fd))
-	{
-		if (3 != sscanf (buf, "%s %s %s", fname, kind, type))
-		{
+	while (Fgets (buf, BUF_LEN, rep_fd)) {
+		if (3 != sscanf (buf, "%s %s %s", fname, kind, type)) {
 			fprintf (stderr, "%s: malformed REPLIES line\n",
 					progname);
-			return (0);
+			return (-1);
 		}
 
 		/* Look for mail or news */
-		if (strcmp(kind,"mail") && strcmp(kind,"news"))
-		{
+		if (strcmp(kind,"mail") && strcmp(kind,"news")) {
 			fprintf (stderr, "%s: bad reply kind: %s\n",
 					progname, kind);
 		}
@@ -347,29 +326,24 @@ SLNPReply ()
 		/* Check reply type */
 		else if ( (type[0] != 'u') &&
 			  (type[0] != 'b') &&
-			  (type[0] != 'B') )
-		{
+			  (type[0] != 'B') ) {
 			fprintf (stderr, "%s: reply type %c not supported\n",
 					progname, type[0]);
-		}
-
-		else
-		{
+		} else {
 			/* Make file name */
 			strcat (fname, ".MSG");
 
 			/* Process it */
-			switch (type[0])
-			{
+			switch (type[0]) {
 			case 'u':
-				if (!strcmp (kind, "mail")) SLuMail (fname);
-				if (!strcmp (kind, "news")) SLuNews (fname);
+				if( !strcmp(kind, "mail")) rval=SLuMail(fname);
+				if( !strcmp(kind, "news")) rval|=SLuNews(fname);
 				break;
 
 			case 'b':
 			case 'B':
-				if (!strcmp (kind, "mail")) SLbMail (fname);
-				if (!strcmp (kind, "news")) SLbNews (fname);
+				if( !strcmp(kind, "mail")) rval=SLbMail(fname);
+				if( !strcmp(kind, "news")) rval|=SLbNews(fname);
 				break;
 			}
 
@@ -377,41 +351,37 @@ SLNPReply ()
 			if (!read_only) unlink (fname);
 		}
 	}
-	return (0);
+	return(rval);
 }
 
-SLuMail (fn)
-char *fn;
 /*
  *  Process a SLNP mail reply file, usenet type
  */
+int SLuMail(char *fn)
 {
 	FILE *fd;
-	int bytes;
-	char *to, *addr, cmd[PATH_LEN];
+	int bytes, ret = 0;
+	char *to, cmd[PATH_LEN];
 	long offset;
 
 	/* Get space for To: */
 	if (NULL == (to = (char *) malloc (BUF_LEN))) OutOfMemory();
 
 	/* Open the reply file */
-	if (NULL == (fd = fopen (fn, "r")))
-	{
+	if (NULL == (fd = fopen (fn, "r"))) {
 		fprintf (stderr, "%s: can't open %s\n", progname, fn);
 		free (to);
-		return (0);
+		return(-1);
 	}
 
 	/* Read through it */
-	while (NULL != Fgets (buf, BUF_LEN, fd))
-	{
-		if (strncmp (buf, "#! rnews ", 9))
-		{
+	while (NULL != Fgets (buf, BUF_LEN, fd)) {
+		if (strncmp (buf, "#! rnews ", 9)) {
 			fprintf (stderr, "%s: malformed reply file\n",
 					progname);
 			fclose (fd);
 			free (to);
-			return (0);
+			return (-1);
 		}
 
 		/* Get byte count */
@@ -424,9 +394,9 @@ char *fn;
 		GetHdr (fd, to, bytes, "To: ");
 
 		/* Check for offline command */
-		if (!strcmp (to, "uqwk") || !strcmp (to, "UQWK"))
-		{
-			OffLine (fd, bytes);
+		if (!strcasecmp (to, "UQWK")) {
+			if((ret = OffLine (fd, bytes)) == -1)
+				break;
 			continue;
 		}
 
@@ -436,95 +406,97 @@ char *fn;
 		printf ("%s: Mailing to %s\n", progname, to);
 
 		/* Pipe message to delivery agent */
-		SLPipe (fd, bytes, cmd);
+		ret = SLPipe(fd, bytes, cmd);
 	}
 	free (to);
 
 	fclose (fd);
-	return (0);
+	return (ret);
 }
 
-SLuNews (fn)
+int SLuNews (fn)
 char *fn;
 /*
  *  Process a SLNP news reply file, usenet type
  */
 {
 	FILE *fd;
-	int bytes;
+	int bytes, rval = 0;
 	char *grp;
 
 	/* Get space for newsgroup name */
-	if (NULL == (grp = (char *) malloc (BUF_LEN)))
-	{
+	if (NULL == (grp = (char *) malloc (BUF_LEN))) {
 		OutOfMemory();
-		return (0);
+		return (-1);
 	}
 
+#ifdef	GNKSA
+	if(gnksa_level &&
+	   GnksaErrors(GnksaCheckFile(fn, fn, gnksa_level&GNKSA_VERBOSE, FROMOPTIONAL))) {
+		AppendBadArticle(fn);
+		if(gnksa_level & GNKSA_HONOUR) {
+			fprintf (stderr, "%s: cannot post invalid message\n", progname);
+			return(-1);
+		} else
+			fprintf (stderr, "%s: warning: attempting to post invalid message\n", progname);
+	}
+#endif
+
 	/* Open the reply file */
-	if (NULL == (fd = fopen (fn, "r")))
-	{
+	if (NULL == (fd = fopen (fn, "r"))) {
 		fprintf (stderr, "%s: can't open %s\n", progname, fn);
-		return (0);
+		return(-1);
 	}
 
 	/* Read through it */
-	while (NULL != Fgets (buf, BUF_LEN, fd))
-	{
-		if (strncmp (buf, "#! rnews ", 9))
-		{
+	while (NULL != Fgets (buf, BUF_LEN, fd)) {
+		if (strncmp (buf, "#! rnews ", 9)) {
 			fprintf (stderr, "%s: mailformed reply file\n",
 					progname);
 			fclose (fd);
-			return (0);
+			return(-1);
 		}
 
 		/* Get byte count */
 		sscanf (&buf[8], "%d", &bytes);
 
-		if (GetHdr (fd, grp, bytes, "Newsgroups: "))
-		{
+		if (GetHdr (fd, grp, bytes, "Newsgroups: ")) {
 			printf ("%s: Posting article to %s\n", progname, grp);
 		}
 
 		/* Pipe message to delivery agent */
-		SLPipe (fd, bytes, SLNP_INEWS);
+		rval = SLPipe(fd, bytes, posting_agent);
 	}
 	free (grp);
 	fclose (fd);
-	return (0);
+	return (rval);
 }
 
-SLbMail (fn)
-char *fn;
 /*
  *  Process a SLNP mail reply file, binary type
  */
+int SLbMail(char *fn)
 {
 	FILE *fd;
-	int bytes;
-	char *to, *addr, cmd[PATH_LEN];
+	int bytes, ret = 0;
+	char *to, cmd[PATH_LEN];
 	long offset;
 
 	/* Get space for To: */
-	if (NULL == (to = (char *) malloc (BUF_LEN))) return (0);
+	if (NULL == (to = (char *) malloc (BUF_LEN))) return (-1);
 
 	/* Open the reply file */
-	if (NULL == (fd = fopen (fn, "r")))
-	{
+	if (NULL == (fd = fopen (fn, "r"))) {
 		fprintf (stderr, "%s: can't open %s\n", progname, fn);
 		free (to);
-		return (0);
+		return(-1);
 	}
 
 	/* Read through it */
-	while (0 != fread (buf, 4, 1, fd))
-	{
+	while (0 != fread (buf, 4, 1, fd)) {
+
 		/* Get byte count */
-		bytes = (buf[0] * 256 * 256 * 256) +
-			(buf[1] * 256 * 256) +
-			(buf[2] * 256) +
-			(buf[3]);
+		bytes = buftoint((unsigned char *) buf);
 
 		/* Remember file position */
 		offset = ftell (fd);
@@ -533,9 +505,9 @@ char *fn;
 		GetHdr (fd, to, bytes, "To: ");
 
 		/* Check for offline command */
-		if (!strcmp (to, "uqwk") || !strcmp (to, "UQWK"))
-		{
-			OffLine (fd, bytes);
+		if (!strcmp (to, "uqwk") || !strcmp (to, "UQWK")) {
+			if((ret=OffLine(fd, bytes)) == -1)
+				break;
 			continue;
 		}
 
@@ -545,61 +517,66 @@ char *fn;
 		printf ("%s: Mailing to %s\n", progname, to);
 
 		/* Pipe message to delivery agent */
-		SLPipe (fd, bytes, cmd);
+		ret = SLPipe(fd, bytes, cmd);
 	}
 	free (to);
 
 	fclose (fd);
-	return (0);
+	return (ret);
 }
 
-SLbNews (fn)
-char *fn;
 /*
  *  Process a SLNP news reply file, binary type
  */
+int SLbNews (char *fn)
 {
 	FILE *fd;
-	int bytes;
+	int  bytes, rval=0;
 	char *grp;
 
 	/* Get space for newsgroup name */
-	if (NULL == (grp = (char *) malloc (BUF_LEN)))
-	{
+	if (NULL == (grp = (char *) malloc (BUF_LEN))) {
 		OutOfMemory();
-		return (0);
+		return (-1);
 	}
 
+#ifdef	GNKSA
+	if(gnksa_level &&
+	   GnksaErrors(GnksaCheckFile(fn, fn, gnksa_level&GNKSA_VERBOSE, FROMOPTIONAL))) {
+		AppendBadArticle(fn);
+		if(gnksa_level & GNKSA_HONOUR) {
+			fprintf (stderr, "%s: cannot post invalid message\n", progname);
+			return(-1);
+		} else
+			fprintf (stderr, "%s: warning: attempting to post invalid message\n", progname);
+	}
+#endif
+
 	/* Open the reply file */
-	if (NULL == (fd = fopen (fn, "r")))
-	{
+	if (NULL == (fd = fopen (fn, "r"))) {
 		fprintf (stderr, "%s: can't open %s\n", progname, fn);
-		return (0);
+		return(-1);
 	}
 
 	/* Read through it */
-	while (0 != fread (buf, 4, 1, fd))
-	{
-		/* Get byte count */
-		bytes = (buf[0] * 256 * 256 * 256) +
-			(buf[1] * 256 * 256) +
-			(buf[2] * 256) +
-			(buf[3]);
+	while (0 != fread (buf, 4, 1, fd)) {
 
-		if (GetHdr (fd, grp, bytes, "Newsgroups: "))
-		{
+		/* Get byte count */
+		bytes = buftoint((unsigned char *) buf);
+
+		if (GetHdr (fd, grp, bytes, "Newsgroups: ")) {
 			printf ("%s: Posting article to %s\n", progname, grp);
 		}
 
 		/* Pipe message to delivery agent */
-		SLPipe (fd, bytes, SLNP_INEWS);
+		rval = SLPipe(fd, bytes, posting_agent);
 	}
 	free (grp);
 	fclose (fd);
-	return (0);
+	return (rval);
 }
 
-SLPipe (fd, bytes, agent)
+int SLPipe (fd, bytes, agent)
 FILE *fd;
 int bytes;
 char *agent;
@@ -608,25 +585,29 @@ char *agent;
  */
 {
 	FILE *pfd;
-	unsigned char c;
+	int c;
 
 	/* Open pipe to agent */
-	if (NULL == (pfd = popen (agent, "w")))
-	{
-		fprintf (stderr, "%s: can't open reply pipe\n", progname);
-		while (bytes--) fgetc (fd);
-		return (0);
+	if (NULL == (pfd = popen (agent, "w"))) {
+		fprintf (stderr, "%s: can't open reply pipe (%s)\n", progname, agent);
+		while (bytes--) getc (fd);
+		return(-1);
 	}
 
 	/* Send message to pipe */
-	while (bytes--)
-	{
-		c = 0xff & fgetc (fd);
-		fputc (c, pfd);
+	while (bytes--) {
+		if( (c = getc(fd) & 0xff) == EOF) {
+			fprintf(stderr,"%s: unable to read %d bytes\n",
+			        progname, bytes);
+			return -1;
+		}
+		if( putc(c, pfd) == EOF) {
+			fprintf(stderr, "%s: write error\n", progname);
+			return -1;
+		}
 	}
 
-	pclose (pfd);
-	return (0);
+	return(pclose(pfd));
 }
 
 int GetHdr (fd, cc, bytes, hdr)
@@ -648,11 +629,9 @@ int bytes;
 	/* Look through header */
 	rc = Fgets (buf, BUF_LEN, fd);
 	n = strlen (buf);
-	while ( (rc != NULL) && (bytes > 0) && (n > 0) )
-	{
+	while ( (rc != NULL) && (bytes > 0) && (n > 0) ) {
 		/* Right line? */
-		if (!strncmp (buf, hdr, cnt))
-		{
+		if (!strncmp (buf, hdr, cnt)) {
 			strcpy (cc, &buf[cnt]);
 			fseek (fd, offset, 0);
 			return (1);
@@ -662,7 +641,7 @@ int bytes;
 		bytes -= n;
 		rc = Fgets (buf, BUF_LEN, fd);
 		if (rc != NULL) n = strlen (buf);
-	}	
+	}
 
 	/* Reposition file */
 	fseek (fd, offset, 0);
@@ -670,7 +649,7 @@ int bytes;
 }
 
 
-DoZipReplies ()
+int DoZipReplies ()
 /*
  *  Process replies in a Zip packet
  */
@@ -678,51 +657,48 @@ DoZipReplies ()
 	int n;
 	char fn[PATH_LEN];
 
-	if (!ZipId()) return (0);
+	if ((n=ZipId()) <=0) return(n);
 
 	/* Loop through possible mail files.  This is a little kludgy,
 	   but readdir() seems to have problems on some systems,
 	   including Esix, which is the system I use. */
-	for (n=0; n<100; n++)
-	{
+	for (n=0; n<100; n++) {
 		/* Construct file name */
 		sprintf (fn, "%s/%s.m%02d", rep_file, user_name, n);
 
 		/* Process it */
-		ZipMail (fn);
+		if((n=ZipMail(fn)) <= 0) return(n);
 	}
 
 	/* Loop through possible news files */
-	for (n=0; n<100; n++)
-	{
+	for (n=0; n<100; n++) {
 		/* Construct file name */
 		sprintf (fn, "%s/%s.n%02d", rep_file, user_name, n);
 
 		/* Process it */
-		ZipNews (fn);
+		if((n=ZipNews(fn)) <= 0) return(n);
 	}
 	return (1);
 }
 
 char ZZZ[PATH_LEN];
 
-ZipId ()
+int ZipId ()
 {
 	char fn[PATH_LEN],zzz[PATH_LEN],zzZ[256],*zZz,ZzZ[PATH_LEN],
 	ZZz[PATH_LEN];FILE *zz;int zZ,zZZ=0,Zzz,z=0;
 
 	sprintf (fn, "%s/%s.id", rep_file, user_name);
-	if (NULL == (zz = fopen (fn, "r")))
-	{
+	if (NULL == (zz = fopen (fn, "r"))) {
 		fprintf (stderr, "%s: can't open %s\n", progname, fn);
-		return (0);
+		return(-1);
 	}
 
 	strcpy(zzz,"Vrth#glfp#YjsMftp-#Kllqbz-");Zzz=strlen(zzz);
-	for(zZz=(&zzz[0]);*zZz;zZz++)*zZz^=3;zZz=(&zzz[0]);zZ=fgetc(zz);
+	for(zZz=(&zzz[0]);*zZz;zZz++)*zZz^=3;zZz=(&zzz[0]);zZ=getc(zz);
 	while(zZ!=EOF){zzZ[z]=zZ^*(zZz+zZZ)^(*zZz*zZZ);*(zZz+zZZ)+=
 	(zZZ<(Zzz+1))?*(zZz+zZZ+1):*zZz;if(!*(zZz+zZZ))(*(zZz+zZZ))++;
-	if(++zZZ>=Zzz)zZZ=0;z++;zZ=fgetc(zz);}zzZ[z]=0;for(z=0;z<9;z++)
+	if(++zZZ>=Zzz)zZZ=0;z++;zZ=getc(zz);}zzZ[z]=0;for(z=0;z<9;z++)
 	zzZ[z]^=3;fclose(zz);if(strncmp(zzZ,"YMQ(VRTH#",9))return(0);
 	zZz=(&zzZ[0]);z=0;zZz=strtok(zZz+9,"\n");zZz=strtok(NULL,"\n");
 	strcpy(ZZz,zZz);zZz=strtok(NULL,"\n");strcpy(ZzZ,zZz);strcat
@@ -736,7 +712,7 @@ ZipId ()
 	return (1);
 }
 
-ZipMail (fn)
+int ZipMail (fn)
 char *fn;
 /*
  *  Process ZipNews mail reply
@@ -744,7 +720,7 @@ char *fn;
 {
 	FILE *fd, *pfd;
 	struct stat stat_buf;
-	int c, have_cc;
+	int c, have_cc, rval = 0;
 	char *to, *cc, *addr;
 
 	/* Get space for To: and Cc: */
@@ -755,20 +731,19 @@ char *fn;
 	if (0 != stat (fn, &stat_buf)) return (0);
 
 	/* Try to open it */
-	if (NULL == (fd = fopen (fn, "r"))) return (0);
+	if (NULL == (fd = fopen (fn, "r"))) return(-1);
 
 	/* Get To: and Cc: */
 	GetHdr (fd, to, stat_buf.st_size, "To: ");
 	have_cc = GetHdr (fd, cc, stat_buf.st_size, "Cc: ");
 
 	/* Check for offline command */
-	if (!strcmp (to, "uqwk") || !strcmp (to, "UQWK"))
-	{
-		OffLine (fd, stat_buf.st_size);
+	if (!strcasecmp (to, "UQWK")) {
+		int ret = OffLine(fd, stat_buf.st_size);
 		free (cc); free (to);
 		fclose (fd);
 		if (!read_only) unlink (fn);
-		return (0);
+		return (ret);
 	}
 
 	/* Make mailer command line */
@@ -777,32 +752,28 @@ char *fn;
 	printf ("%s: Mailing to %s\n", progname, to);
 
 	/* Open pipe to mailer */
-	if (NULL == (pfd = popen (buf, "w")))
-	{
+	if (NULL == (pfd = popen (buf, "w"))) {
 		fprintf (stderr, "%s: can't popen() mailer\n", progname);
 		free (cc); free (to);
 		fclose (fd);
-		return (0);
+		return (-1);
 	}
 
-	fprintf (pfd, "%s\n", ZZZ);
+	if( (fprintf (pfd, "%s\n", ZZZ) < 0) && ferror(pfd)) return(-1);
 
 	/* Send bytes of message */
-	while (EOF != (c = fgetc (fd)))
-	{
-		if (c != '\r') fputc ((0xff & c), pfd);
+	while (EOF != (c = getc (fd))) {
+		if (c != '\r') putc ((0xff & c), pfd);
 	}
 
 	/* Done */
-	pclose (pfd);
+	rval = pclose (pfd);
 
 	/* Now do Cc: addresses */
-	if (have_cc)
-	{
+	if (have_cc) {
 		addr = strtok (cc, ", \t");
 
-		while (addr != NULL)
-		{
+		while (addr != NULL) {
 			/* Rewind file */
 			fseek (fd, 0, 0);
 
@@ -812,24 +783,22 @@ char *fn;
 			printf ("%s:  Cc'ing to %s\n", progname, addr);
 
 			/* Open pipe to mailer */
-			if (NULL == (pfd = popen (buf, "w")))
-			{
+			if (NULL == (pfd = popen (buf, "w"))) {
 				fprintf (stderr, "%s: can't popen() mailer\n",
 						progname);
 				fclose (fd);
-				return (0);
+				return(-1);
 			}
 
-			fprintf (pfd, "%s\n", ZZZ);
+			if( (fprintf (pfd, "%s\n", ZZZ) < 0) && ferror(pfd)) return(-1);
 
 			/* Send bytes of message */
-			while (EOF != (c = fgetc (fd)))
-			{
-				if (c != '\r') fputc ((0xff & c), pfd);
+			while (EOF != (c = getc (fd))) {
+				if (c != '\r') putc ((0xff & c), pfd);
 			}
 
 			/* Done */
-			pclose (pfd);
+			rval |= pclose (pfd);
 
 			addr = strtok (NULL, ", \t");
 		}
@@ -839,10 +808,10 @@ char *fn;
 	fclose (fd);
 	if (!read_only) unlink (fn);
 
-	return (1);
+	return (rval);
 }
 
-ZipNews (fn)
+int ZipNews (fn)
 char *fn;
 /*
  *  Process ZipNews news reply
@@ -850,37 +819,76 @@ char *fn;
 {
 	FILE *fd, *pfd;
 	struct stat stat_buf;
-	int c;
+	int c, rval = 0;
 
 	/* Try to stat() it */
 	if (0 != stat (fn, &stat_buf)) return (0);
 
+#ifdef	GNKSA
+	if(gnksa_level &&
+	   GnksaErrors(GnksaCheckFile(fn, fn, gnksa_level&GNKSA_VERBOSE, FROMOPTIONAL))) {
+		AppendBadArticle(fn);
+		if(gnksa_level & GNKSA_HONOUR) {
+			fprintf (stderr, "%s: cannot post invalid message\n", progname);
+			return(-1);
+		} else
+			fprintf (stderr, "%s: warning: attempting to post invalid message\n", progname);
+	}
+#endif
+
 	/* Try to open it */
-	if (NULL == (fd = fopen (fn, "r"))) return (0);
+	if (NULL == (fd = fopen (fn, "r"))) return(-1);
+
 
 	printf ("%s: Posting article...\n", progname);
 
 	/* Open pipe to inews */
-	if (NULL == (pfd = popen (ZIP_INEWS, "w")))
-	{
+	if (NULL == (pfd = popen (posting_agent, "w"))) {
 		fprintf (stderr, "%s: can't popen() inews\n", progname);
 		fclose (fd);
-		return (0);
+		return(-1);
 	}
 
-	fprintf (pfd, "%s\n", ZZZ);
+	if( (fprintf (pfd, "%s\n", ZZZ) < 0) && ferror(pfd)) return(-1);
 
 	/* Send bytes of message */
-	while (EOF != (c = fgetc (fd)))
-	{
-		if (c != '\r') fputc ((0xff & c), pfd);
+	while (EOF != (c = getc (fd))) {
+		if (c != '\r') putc ((0xff & c), pfd);
 	}
 
 	/* Done */
-	pclose (pfd);
+	rval = pclose (pfd);
 	fclose (fd);
 	if (!read_only) unlink (fn);
 
-	return (1);
+	return (rval);
 }
 
+#ifdef	GNKSA
+
+void AppendBadArticle(char *fn)
+{
+	char   *buf, out_file[PATH_LEN];
+	FILE   *fd;
+	size_t size;
+
+	sprintf(out_file, "%s/dead.articles", home_dir);
+
+	if((fd = fopen(out_file, "a")) == NULL)
+		fprintf(stderr, "could not open %s for appending\n", out_file);
+
+	if((buf = SlurpFile(fn, &size)) == NULL) {
+		fprintf(stderr, "could not get bad article %s\n", fn);
+		fclose(fd);
+		return;
+	}
+
+	if(fwrite(buf, 1, size, fd) != size)
+		fprintf(stderr, "failed to write %i bytes to %s\n", size, fn);
+
+	fclose(fd);
+	free(buf);
+
+	fprintf(stderr, "%s: bad article appended to %s\n", progname, out_file);
+}
+#endif
